@@ -175,6 +175,7 @@ function is_video($file) {
 	if (	!preg_match("/^[\._]/", $file) &&
 		(preg_match("/\.mpg$/i", $file) ||
 		preg_match("/\.mpeg$/i", $file) ||
+		preg_match("/\.mp4$/i", $file) ||
 		preg_match("/\.mov$/i", $file) ||
 		preg_match("/\.avi$/i", $file))
 	) {
@@ -261,6 +262,30 @@ function do_resize_picture($path_to_picture, $width, $height, $rotate) {
 /****************************************************************************/
 
 
+/****************************************************************************/
+/* Transcode a video, returns temp file name, depends on libav-tools */
+function do_transcode_video($path) {
+	$path_parts = preg_split("/\//", $path);
+	$file = array_pop($path_parts);
+        $tempfilename = get_cache_filename($file, "transcode");
+	if (is_video($path)) {
+		if (preg_match("/.*\.mp4$/i", $path)) {
+			return "$path";
+		} elseif (! file_exists($tempfilename)) {
+			$input = escapeshellarg($path);
+			try {
+				print("<meta http-equiv='refresh' content='20'>");
+				shell_exec("HOME=/var/cache/pictor/ run-one avconv -i " . escapeshellarg("$path") . " -vcodec libx264 -strict experimental " . escapeshellarg("$tempfilename"));
+			} catch (Exception $e) {
+				$tempfilename = "";
+			}
+		}
+		//clean_tmp("tmp/transcode");
+		return $tempfilename;
+	}
+}
+/****************************************************************************/
+
 /****************************************************************************/
 /* Find the index of a given value in an array */
 function locate_index($needle, $haystack) {
@@ -311,6 +336,16 @@ body {
 	font-weight: 400;
 	color: black;
 }
+div {
+	position:relative;
+	display:inline;
+}
+.vid {
+	position:absolute;
+	top:0;
+	right:0;
+}
+</style>
 </style>
 <title>$TITLE</title>
 <link rel='shortcut icon' href='/pictor/favicon.ico' type='image/x-icon'>
@@ -367,7 +402,11 @@ function get_cache_filename($filename, $dir) {
 	$md5 = md5($filename);
 	$cache_filename = "tmp/$dir/" . substr($md5, 0, 2);
 	@mkdir($cache_filename);
-	return "$cache_filename/$md5.jpg";
+	if ("$dir" == "transcode") {
+		return "$cache_filename/$md5.mp4";
+	} else {
+		return "$cache_filename/$md5.jpg";
+	}
 }
 
 /****************************************************************************/
@@ -379,9 +418,9 @@ function print_thumbnail($path, $file, $desc) {
 	$thumbnail_name = get_cache_filename($filename, "thumbnails");
 	$href = "?album=" . urlencode($path) . "&picture=" . urlencode($file);
 	print("<a title='$desc' href='$href'>");
-	if (is_image_filename($filename)) {
-		if (! file_exists($thumbnail_name)) {
-			// No thumbnail in cache.
+	if (! file_exists($thumbnail_name)) {
+		// No thumbnail in cache.
+		if (is_image_filename($filename)) {
 			if ($img = @exif_thumbnail($filename)) {
 				// Try to extract thumbnail from the image.
 				$fh = fopen($thumbnail_name, "w");
@@ -391,7 +430,7 @@ function print_thumbnail($path, $file, $desc) {
 				try {
 					// Otherwise, try Imagick.
 					$img = new Imagick($filename);
-					$img->scaleImage(150, 150, true);
+					$img->scaleImage(130, 130, true);
 					$img->writeImage($thumbnail_name);
 					$img->destroy();
 				} catch (Exception $e) {
@@ -399,10 +438,15 @@ function print_thumbnail($path, $file, $desc) {
 				}
 			}
 			rotate_if_necessary($filename, $thumbnail_name);
+		} elseif (is_video($filename)) {
+			shell_exec("HOME=/var/cache/pictor/ run-one avconv -ss 00:00:00 -i " . escapeshellarg("$filename") . " -vsync 1 -t 0.01 " . escapeshellarg("$thumbnail_name"));
 		}
-		print("<img height=130 align=center border=0 src='$thumbnail_name'></a>&nbsp;");
-	} elseif (is_video($file)) {
-		print("<img width=32 src=silk/film.png>");
+	} else {
+		print("<div><img height=130 align=center border=0 src='$thumbnail_name'>");
+		if (is_video($filename)) {
+			print("<img width=64 src='silk/resultset_next.png' class='vid'>");
+		}
+		print("</div></a>\n");
 	}
 }
 /****************************************************************************/
@@ -766,7 +810,8 @@ if (window.innerHeight > window.innerWidth) {
 </script>
 ");
 	} elseif (is_video($path_to_picture)) {
-		print("Play <embed src='$path_to_picture' name='Video clip' loop='false' cache='true' width=400 height=300 controller='true' autoplay='true'></embed>");
+		$tempvideoname = do_transcode_video($path_to_picture);
+		print("<video width=400 height=300 controls><source src='$tempvideoname' type='video/mp4'></video>");
 	}
 	print("</a>");
 }
